package fr.michaelm.utils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.util.jar.*;
import java.util.zip.ZipEntry;
import java.util.HashMap;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class JarUpdater extends Thread {
    private File f0, f1;
    private JarInputStream jis0 = null;
    private JarInputStream jis1 = null;
    private JarOutputStream jos = null;
    private JTextArea jta = new JTextArea(10,30);
    
    private final byte[] buffer = new byte[4096];
    
    public JarUpdater() {}
    
    public static void main(String[] args) {
        JarUpdater ju = new JarUpdater();
        try {
            ju.initGui();
        } catch(IOException ex) {ex.printStackTrace();}
    }
    
    private void initGui() throws IOException {
        final JDialog dialog = new JDialog(new JFrame(), "jar updating");
        final JTextField jtf0 = new JTextField("jartobeupdated.jar", 12);
        final JTextField jtf1 = new JTextField("patch.jar", 12);
        final JPanel northPanel = new JPanel();
        northPanel.setLayout(new GridLayout(3,2));
        northPanel.add(new JLabel("Jar file to be updated"));
        northPanel.add(jtf0);
        northPanel.add(new JLabel("Jar containing new classes"));
        northPanel.add(jtf1);
        dialog.getContentPane().add(northPanel, BorderLayout.NORTH);
        dialog.getContentPane().add(new JScrollPane(jta), BorderLayout.CENTER);
        dialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
        final JButton jbOK = new JButton("Valider");
        jbOK.addActionListener(new java.awt.event.ActionListener(){
            public void actionPerformed(ActionEvent e) {
                f0 = new File(jtf0.getText());
                if (!f0.exists()) {
                    //System.out.println(f0.getName() + " is not a valid file name");
                    jta.append(f0.getName() + " is not a valid file name\n");
                    return;
                }
                f1 = new File(jtf1.getText());
                if (!f1.exists()) {
                    //System.out.println(f1.getName() + " is not a valid file name");
                    jta.append(f1.getName() + " is not a valid file name\n");
                    return;
                }
                try {
                    jis0 = new JarInputStream(new FileInputStream(f0));
                    jis1 = new JarInputStream(new FileInputStream(f1));
                    jos = new JarOutputStream(new FileOutputStream(new File(f0.getAbsolutePath()+".tmp")));
                    //update();
                    start();
                }
                catch(Exception ex){jta.append(ex.getMessage());}
            }
        });
        final JButton jbCANCEL = new JButton("Quitter");
        jbCANCEL.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
                System.exit(0);
            }
        });
        northPanel.add(jbOK);
        northPanel.add(jbCANCEL);
        dialog.pack();
        dialog.validate();
        dialog.setVisible(true);
    }
    
    public void run() {
        try {update();}
        catch(IOException e) {jta.append("ERREUR : " + e.getMessage() + "\n");}
    }
    
    public void update() throws IOException {
        
        HashMap mapsrc = new HashMap();
        HashMap mapEntries = new HashMap();
        
        ZipEntry entry;
        jta.append("Lecture du fichier jar initial...\n");
        while ((entry = jis0.getNextEntry()) != null) {
            String name = entry.getName();
            //jta.append(name+"...\n");
            long time = entry.getTime();
            mapsrc.put(name, "0");
            mapEntries.put(name, entry);
        }
        jta.append("Lecture du fichier jar de mise  jour...\n");
        while ((entry = jis1.getNextEntry()) != null) {
            String name = entry.getName();
            long time = entry.getTime();
            if(mapsrc.containsKey(name)) {
                if (time>((ZipEntry)mapEntries.get(name)).getTime()) {
                    //jta.append(name+"...\n");
                    mapsrc.put(name, "1");
                }
            }
            else if(!name.endsWith("MF")) {
                mapsrc.put(name, "1");
            }
            else;
        }
        jta.append("Fermeture des deux fichiers jars...\n");
        jis0.close();
        jis1.close();
        jta.append("Deux fichiers ferms\n");
        
        jis0 = new JarInputStream(new FileInputStream(f0));
        while ((entry = jis0.getNextEntry()) != null) {
            String name = entry.getName();
            if (mapsrc.get(name).equals("0")) {
                addFile(name, jis0, jos);
            }
        }
        jis1 = new JarInputStream(new FileInputStream(f1));
        while ((entry = jis1.getNextEntry()) != null) {
            String name = entry.getName();
            if (name.endsWith("MF")) continue;
            if (mapsrc.get(name).equals("1")) {
                jta.append("Ajout du fichier : " + name + "\n");
                jta.setCaretPosition(jta.getText().length());
                addFile(name, jis1, jos);
            }
        }
        jta.append("Fermeture du fichier jar source...\n");
        jis0.close();
        jta.append("Fichier jar source ferm\n");
        jta.append("Fermeture du fichier jar de mise  jour...\n");
        jis1.close();
        jta.append("Fichier de mise  jour ferm\n");
        jta.append("Fermeture du fichier tampon...\n");
        jos.close();
        jta.append("Fichier tampon ferm\n");
        
        String sf0 = f0.getAbsolutePath();
        String oldf0 = sf0+".old";
        jta.append("Renommage du fichier jar en jar.old\n");
        f0.renameTo(new File(oldf0));
        jta.append("Renommage du fichier tmp en fichier jar\n");
        new File(sf0+".tmp").renameTo(new File(sf0));
        jta.append("FIN\n");
        jta.setCaretPosition(jta.getText().length());
    }
            
            
    public void addFile(String path, InputStream source, JarOutputStream jos) throws IOException {
         jos.putNextEntry(new ZipEntry(path));
         try {
             int count;
             while ((count = source.read(buffer)) > 0) {
                 jos.write(buffer, 0, count);
             }
         } finally {
             jos.closeEntry();
         }
     }

}
